#include "config.h"

#include <stdio.h>
#include <string.h>	/* memcpy/memset, try to remove */
#include <stdlib.h>

#include "mpeg2.h"
#include "mm_accel.h"
#include "attributes.h"
#include "mmx.h"
#include "malloc.h"

extern FILE* log;

mpeg2_config_t config;

//---------------------------------------------------------
//   Mpeg2dec
//---------------------------------------------------------

Mpeg2dec::Mpeg2dec(uint32_t mm_accel)
      {
      config.flags = mm_accel;
      picture      = (picture_t*)memalign (16, sizeof (picture_t));
      memset(picture, 0, sizeof (picture_t));
      frames       = 0;
      frame_ptr[0] = 0;
      frame_ptr[1] = 0;
      frame_ptr[2] = 0;
      idct_init();
      motion_comp_init();
      }

Mpeg2dec::~Mpeg2dec()
      {
      free(picture);
      if (frames)
            free(frames);
      }

//---------------------------------------------------------
//   frameDone
//---------------------------------------------------------

void Mpeg2dec::frameDone()
      {
      if (((picture->picture_structure == FRAME_PICTURE) || (picture->second_field))) {
            Frame* frame = (picture->picture_coding_type == B_TYPE) ?
               picture->current_frame : picture->forward_reference_frame;
            if (config.flags & MM_ACCEL_X86_MMX)
                  emms();
            draw(frame);
            }
      }

//---------------------------------------------------------
//   startSequence
//---------------------------------------------------------

void Mpeg2dec::startSequence()
      {
      picture->header_state_init();
      int size  = width() * height() / 4;
      if (frames)
            free(frames);
      frames = (unsigned char*)memalign(64, size*18);
      unsigned char* alloc = frames;

      for (int i = 0; i < 3; i++) {
            Frame* f     = new Frame;
            frame_ptr[i] = f;
            f->base[0]   = alloc;
            f->base[2]   = alloc + 4 * size;
            f->base[1]   = alloc + 5 * size;
            alloc       += 6 * size;
            f->pts = 0;
            }
      picture->forward_reference_frame  = frame_ptr[0];
      picture->backward_reference_frame = frame_ptr[1];
      picture->current_frame            = frame_ptr[2];
      prediction_index                  = 1;
      }

//---------------------------------------------------------
//   startSlice
//---------------------------------------------------------

extern uint64_t vpts;

void Mpeg2dec::startSlice()
      {
      if (picture->second_field)
            picture->current_frame->field(picture->picture_structure);
      else {
            if (picture->picture_coding_type == B_TYPE)
                  picture->current_frame = frame_ptr[2];
            else {
                  picture->current_frame            = frame_ptr[prediction_index ? 0: 1];
                  prediction_index                 ^= 1;
                  picture->forward_reference_frame  = picture->backward_reference_frame;
                  picture->backward_reference_frame = picture->current_frame;
                  }
            }
      picture->current_frame->pts = vpts;
      }

//---------------------------------------------------------
//   processSlice
//---------------------------------------------------------

void Mpeg2dec::processSlice(unsigned char code, unsigned char* buffer)
      {
      slice_process(picture, code, buffer);
      if (config.flags & MM_ACCEL_X86_MMX)
            emms();
      }

//---------------------------------------------------------
//   frameRate
//---------------------------------------------------------

void Mpeg2dec::frameRate(int& z, int& n) const
      {
      z = 0;
      n = 1;
      switch(frameRateCode()) {
            case 0:
            default:
                  break;
            case 1:
                  z = 24000;
                  n = 1001;
                  break;
            case 2:
                  z = 24;
                  break;
            case 3:
                  z = 25;
                  break;
            case 4:
                  z = 30000;
                  n = 1001;
                  break;
            case 5:
                  z = 30;
                  break;
            case 6:
                  z = 50;
                  break;
            case 7:
                  z = 60000;
                  n = 1001;
                  break;
            case 8:
                  z = 60;
                  break;
            }
      }

//---------------------------------------------------------
//   aspect
//---------------------------------------------------------

void Mpeg2dec::aspect(int& z, int& n) const
      {
      switch(aspectRatio()) {
            case 2:
                  z = 4;
                  n = 3;
                  break;
            case 3:
                  z = 16;
                  n = 9;
                  break;
            case 4:
                  z = 221;
                  n = 100;
                  break;
            }
      }

